

# pairwise (dis)similarity heatmaps

pwmap <- function( C, dlab=NULL, col=rb.colors, saturation=3, gutter = 0.5,
  rdw = 3, cdw = 3, stratify = TRUE, hang=.05 )
{
  par0 <- par( no.readonly=TRUE )
  on.exit( par(par0) )

  if( is.null(C$subclust) ) stratify <- FALSE

  if( is.null(C$pwcorr) )
    stop("pairwise correlation matrix not present. rerun nclust2 with rpwcorr=TRUE")

  K <- if( stratify ) length(C$subclust) else 1

  L <- rbind(
          rep(0,K+2),
          cbind(rep(0,K),array( 1:(K*K),dim=c(K,K)),rep(0,K)),
          rep(0,K+2))

  L[K+2,1 + (1:K)] <- (K*K)+(1:K)
  L[1+(1:K),1] <- K*(K+1) + (1:K)

  layout( L, widths=c(lcm(rdw),rep(1,K),lcm(2)),
    heights=c(lcm(2),rep(1,K),lcm(cdw)) )
  
  for(i in K:1)
    {
    Si <- if( stratify ) C$subclust[[i]] else C
    li <- Si$labels[ Si$order ]
    for(j in 1:K )
      {
      Sj <- if( stratify ) C$subclust[[j]] else C
      
      par(mar = c(gutter,gutter,gutter,gutter))

      lj <- Sj$labels[ Sj$order ]

      rres <- length(lj) %/% 500 + 1
      cres <- length(li) %/% 500 + 1
      dx <- t( downsample( C$pwcorr[ lj, li ], rres, cres,
        rcenter=FALSE, ccenter = FALSE ) )

      image( tanh( saturation*(dx) ),
        zlim=c(-1,1), col=col,
        x = seq( 1, length(li),rres ),
        y = seq( 1, length(lj),cres ),
        axes=FALSE
        )
      }
    }

  rdend <- dendrolines(C,hang=hang,height=-C$branch.level)
  rdend.range <- range(-rdend$y)

  for(i in K:1)
    {
    subclust <- if(stratify) C$subclust[[i]] else C
    u <- dendrolines( subclust,
      hang=hang, 
      height= -if(stratify)
                C$branch.level[subclust$sup.branch]
               else
                C$branch.level
      )
    par(mar=c(2,gutter,1,gutter))
    plot( u$x, -u$y, type="l",axes=FALSE,ann=FALSE, xaxs="i",
      ylim=rdend.range )
    if(!is.null(dlab))
      {
      lab <- if(stratify) dlab[i] else do.call(paste,as.list(c(dlab,sep=":")))
      mtext( lab, side=1,line=.5,at=length(subclust$labels)/2,cex=.8)
      }
    }

  for(i in 1:K)
    {
    subclust <- if(stratify) C$subclust[[i]] else C
    u <- dendrolines( subclust,
      hang=hang, height= -if(stratify)
        C$branch.level[subclust$sup.branch] else C$branch.level )
    par(mar=c(gutter,2,gutter,1))
    plot( -u$y, u$x, type="l",axes=FALSE,ann=FALSE, yaxs="i",
      xlim=rdend.range )
    if(!is.null(dlab))
      {
      lab <- if(stratify) dlab[i] else do.call(paste,as.list(c(dlab,sep=":")))
      mtext( lab, side=2,line=.5,at=length(subclust$labels)/2,cex=.8)
      }
    }
}

